# Fourier-wavelet Software

[https://igit.iap-kborn.de/yamazaki/fourierwavelet](https://igit.iap-kborn.de/yamazaki/fourierwavelet)

This software is created by [Y. Yamazaki](https://orcid.org/0000-0002-7624-4752) at the Leibniz Institute of Atmospheric Physics (IAP) at the University of Rostock, Schlossstraße 6, 18225 Kühlungsborn, Germany, and is published on 14 November 2022. It computes Fourier-wavelet spectra for given longitude-time data. A Fourier-wavelet spectrum is similar to a wavelet spectrum, but is given separately for eastward- and westward-propagating waves with different zonal wavenumbers.

The method involves the Fourier analysis in longitude and wavelet analysis in time. The Fourier analysis is based on the fast Fourier transform (FFT) or the least-squares method, depending on whether there are gaps in the data. The FFT is used when there is no missing value (or ```NaN```) in the data, while the least-squares method is used when the data contain ```NaN```. The wavelet analysis is based on the method described by [Torrence and Compo (1998)](https://journals.ametsoc.org/view/journals/bams/79/1/1520-0477_1998_079_0061_apgtwa_2_0_co_2.xml). The software created by Torrence and Compo (1998) is avaialbe from [here](https://atoc.colorado.edu/research/wavelets/) and is already included in this package.

## Citation

Reference: Yamazaki, Y. (2023): Matlab and Python software to compute Fourier-wavelet spectra (fourierwavelet v1.1) using longitude-time data for studying global-scale atmospheric waves

## Version

Version 1.0 (released on 14 November 2022)

* The initial software was released as "[cfw](https://zenodo.org/record/7458051#.ZCwcbOxBz0o)".

Version 1.1 (released on 22 March 2023)

* The software name was changed to "fourierwavelet".

## Matlab Code

The main Matlab function to compute Fourier-wavelet spectra is ```fourierwavelet.m```. It uses the wavelet functions provided by Torrence and Compo (i.e., ```wavelet.m```, ```wave_bases.m```, ```wave_signif.m```, ```chisquare_inv.m```, ```chisquare_solve.m```), which are all included in this Fourier-wavelet package.

```
[Rw,Re,period,coi,signif,Pw,Pe] = fourierwavelet(data,dt,dlon,zwn,min_t,max_t,dj)
```

```[Rw,Re,period] = fourierwavelet(data,dt,dlon,zwn)``` returns the amplitude of the eastward- and westward-propagating wave components and their wave periods.

```[ ___ ,coi,signif,Pw,Pe] = fourierwavelet( ___ )``` returns the cone of influence and the threshold for the 95% significance level, as well as the phase of the eastward- and westward-propagating wave components.

```[ ___ , period, ___ ] = fourierwavelet( ___ ,min\_t,max\_t,dj)``` specifies the maximum and minimum values of the period to be output, as well as specifies the scale resolution.

### inputs

```data``` is a space-time data matrix [time x longitude]. Each column corresponds to a time series at a fixed longitude. Each row corresponds to a longitude distribution at a fixed time. Make sure that ```size(data,1)``` gives the length in time, and ```size(data,2)``` gives the length in longitude. The data need to be equally distributed in time and longitude. Gaps should be filled by ```NaN```. Also, there should  be no repetition in longitude. For example, if the first and last columns are assigned to the same longitude (e.g., 0 and 360 deg), remove the last column, i.e., ```data = data(:,end-1)```, to avoid the repetition.

```dt``` is sampling time (arbitrary units).

```dlon``` is longitude spacieng (in units of degrees).

```zwn``` is the zonal wavenumber (an integer value, >=0).

### outputs

```Rw``` is a matrix [```period``` x time] of the amplitude of the westward-propagating wave component with zonal wavenuber ```zwn```.

```Re``` is the same as ```Rw``` except for the eastward-propagating wave component.

```period``` is a vector of Fourier periods (in time units).

```coi``` is a vector for the Cone-of-Influence. It gives a vector that contains the maximum period of useful information at that particular time. Periods greater than ```coi``` are subject to edge effects. This can be used to plot ```coi``` lines on a contour plot, for example:

```
contour(time,log2(period),Rw) 
hold on
plot(time,log2(coi),'k')
```

```signif``` is a matrix [```period``` x time] in the same size as ```Rw``` and ```Re```. ```Rw``` (or ```Re```) is above the 95% confidence for a red noise background (lag-1 autocorrelation = 0.72) where ```Rw``` (or ```Re```) > ```signif```. This can be used to plot the 95% confidence lines, for example:

```
contour(time,log2(period),Rw./signif,1,'m')
```

```Pw``` is a matrix [```period``` x time] of the phase of the westward-propagating wave component with zonal wavenumber ```zwn```.

```Pe``` is the same as ```Rw``` except for the eastward-propagating wave component.

### optional inputs

Note: Setting any of the following to -1 will cause the default value to be used.

```min_t``` and ```max_t``` specify the minimum and minimum ```period``` to be included, in the same time units as ```dt```. If not specified, the results are output for full ```period``` range.

```dj``` is the spacing between discrete scales. Default is 0.1. A smaller value will give better scale resolution, but the computation takes more time.

### test

A test script ```fourierwavelet_test.m``` is included. 

```
fourierwavelet_test
```

It reads example data ```SD-WACCM-X_v2.1_GPH.dat```, performs ```fourierwavelet```, and plots out the results. 

<img src="demo_plot.png"  width="520">


Details of the SD/WACCM-X simulation for ```SD-WACCM-X_v2.1_GPH.dat``` can be found in [Gasperini et al. (2020)](https://agupubs.onlinelibrary.wiley.com/doi/full/10.1029/2019ja027649).

<hr style="border:2px solid gray">

## Python Code

The main Python function to compute Fourier-wavelet spectra is ```fourierwavelet.py```. It uses the wavelet functions provided by Torrence and Compo (i.e., ```waveletAnalysis.py``` and ```waveletFunctions.py```), which are all included in this Fourier-wavelet package. It also uses [SciPy](https://scipy.org/) and [NumPy](https://numpy.org/) libraries, which are not included in this package.

```
Rw,Re,period,coi,signif,Pw,Pe = fourierwavelet(data,dt,dlon,zwn,min_t,max_t,dj)
```
This computes the amplitude and phase of the eastward- and westward-propagating wave components, wave periods, the cone of influence, and the threshold for the 95% significance level.

### inputs

```data``` (*numpy array*) is a space-time data matrix [time x longitude]. Each column corresponds to a time series at a fixed longitude. Each row corresponds to a longitude distribution at a fixed time. Make sure that ```data.shape[0]``` gives the length in time, and ```data.shape[1]``` gives the length in longitude. The data need to be equally distributed in time and longitude. Gaps should be filled by ```numpy.nan```. Also, there should  be no repetition in longitude. For example, if the first and last columns are assigned to the same longitude (e.g., 0 and 360 deg), remove the last column, i.e., ```data = data[:,0:-1]```, to avoid the repetition.

```dt``` is sampling time (arbitrary units).

```dlon``` is longitude spacieng (in units of degrees).

```zwn``` is the zonal wavenumber (an integer value, >=0).

### outputs

```Rw``` (*numpy array*) is a matrix [```period``` x time] of the amplitude of the westward-propagating wave component with zonal wavenuber ```zwn```.

```Re``` (*numpy array*) is the same as ```Rw``` except for the eastward-propagating wave component.

```period``` (*numpy array*) is a vector of Fourier periods (in time units).

```coi``` (*numpy array*) is a vector for the Cone-of-Influence. It gives a vector that contains the maximum period of useful information at that particular time. Periods greater than ```coi``` are subject to edge effects. This can be used to plot ```coi``` lines on a contour plot, for example:

```
matplotlib.pylab.contour(time,numpy.log2(period),Rw)
matplotlib.pylab.plot(time,numpy.log2(coi),'k')
```

```signif``` (*numpy array*) is a matrix [```period``` x time] in the same size as ```Rw``` and ```Re```. ```Rw``` (or ```Re```) is above the 95% confidence for a red noise background (lag-1 autocorrelation = 0.72) where ```Rw``` (or ```Re```) > ```signif```. This can be used to plot the 95% confidence lines, for example:

```
matplotlib.pylab.contour(time,numpy.log2(period),Rw/signif,1,'m')
```

```Pw``` (*numpy array*) is a matrix [```period``` x time] of the phase of the westward-propagating wave component with zonal wavenumber ```zwn```.

```Pe``` (*numpy array*) is the same as ```Rw``` except for the eastward-propagating wave component.

### optional inputs

```min_t``` and ```max_t``` specify the minimum and minimum ```period``` to be included, in the same time units as ```dt```. If not specified, the results are output for full ```period``` range.

```dj``` is the spacing between discrete scales. Default is 0.1. A smaller value will give better scale resolution, but the computation takes more time.

### test

A test script ```fourierwavelet_test.py``` is included. 

```
fourierwavelet_test
```

It reads example data ```SD-WACCM-X_v2.1_GPH.dat```, performs ```fourierwavelet```, and plots out the results. (See the figure above.) Details of the SD/WACCM-X simulation for ```SD-WACCM-X_v2.1_GPH.dat``` can be found in [Gasperini et al. (2020)](https://agupubs.onlinelibrary.wiley.com/doi/full/10.1029/2019ja027649).


## License
The Fourier-wavelet software (```fourierwavelet.m```, ```fourierwavelet_test.m```, ```fourierwavelet.py``` and ```fourierwavelet_test.py```) are licensed under the [GNU General Public License](https://www.gnu.org/licenses/).

The wavelet software by Torrence and Compo (1998) are licensed under the [MIT License](https://github.com/chris-torrence/wavelets/blob/main/LICENSE).
